@@ -1,4 +1,7 @@
-/* Copyright (c) 1993,1994, Joseph Arceneaux.  All rights reserved.
+/*
+   Copyright 1999-2002,2010, Thomas E. Dickey
+
+   Copyright (c) 1993,1994, Joseph Arceneaux.  All rights reserved.
 
    This file is subject to the terms of the GNU General Public License as
    published by the Free Software Foundation.  A copy of this license is
@@ -48,51 +51,39 @@
 #include "sys.h"
 #include "backup.h"
 #include "indent.h"
-#include <ctype.h>
-
-#include <stdlib.h>
-#if defined (HAVE_UNISTD_H)
-#include <unistd.h>
-#endif
 
 #ifdef _WIN32
 #include <io.h>
 #else
 #include <fcntl.h>
 #endif
-#include <string.h>
-
 
 #ifndef isascii
-#define ISDIGIT(c) (isdigit ((unsigned char) (c)))
+#define ISDIGIT(c) (isdigit (UChar(c)))
 #else
-#define ISDIGIT(c) (isascii (c) && isdigit (c))
+#define ISDIGIT(c) (isascii (UChar(c)) && isdigit (UChar(c)))
 #endif
 
 #ifndef NODIR
 
 #include <sys/types.h>
 
-#ifdef DIRENT
-#include <dirent.h>
-#ifdef direct
-#undef direct
+#if HAVE_DIRENT_H
+# include <dirent.h>
+# define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
+# define dirent direct
+# define NAMLEN(dirent) (dirent)->d_namlen
+# if HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# if HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+# if HAVE_NDIR_H
+#  include <ndir.h>
+# endif
 #endif
-#define direct dirent
-#define NLENGTH(direct) (strlen((direct)->d_name))
-#else /* !DIRENT */
-#define NLENGTH(direct) ((direct)->d_namlen)
-#ifdef USG
-#ifdef SYSNDIR
-#include <sys/ndir.h>
-#else /* !SYSNDIR */
-#include <ndir.h>
-#endif /* !SYSNDIR */
-#else /* !USG */
-#include <sys/dir.h>
-#endif /* !USG */
-#endif /* !DIRENT */
-
 
 #if defined (_POSIX_VERSION)	/* Might be defined in unistd.h.  */
 /* POSIX does not require that the d_ino field be present, and some
@@ -118,10 +109,8 @@
 #define BACKUP_SUFFIX_FORMAT "%s.~%d~"
 #endif
 
-extern void free ();
-
 /* Default backup file suffix to use */
-static char *simple_backup_suffix = BACKUP_SUFFIX_STR;
+static const char *simple_backup_suffix = BACKUP_SUFFIX_STR;
 
 /* What kinds of backup files to make -- see
    table `version_control_values' below. */
@@ -132,8 +121,7 @@ enum backup_mode version_control = unknown;
    the value of `simple_backup_suffix'. */
 
 static char *
-simple_backup_name (pathname)
-     char *pathname;
+simple_backup_name (char *pathname)
 {
   char *backup_name;
 
@@ -148,13 +136,10 @@ simple_backup_name (pathname)
    that number.  BASE_LENGTH is the string length of BASE. */
 
 static int
-version_number (base, direntry, base_length)
-     char *base;
-     char *direntry;
-     int base_length;
+version_number (const char *base, const char *direntry, size_t base_length)
 {
   int version;
-  char *p;
+  const char *p;
 
   version = 0;
   if (!strncmp (base, direntry, base_length)
@@ -174,34 +159,33 @@ version_number (base, direntry, base_length)
    DIRNAME.  Return 0 if there are no numbered versions. */
 
 static int
-highest_version (filename, dirname)
-     char *filename, *dirname;
+highest_version (const char *filename, const char *dirname)
 {
   DIR *dirp;
-  struct direct *dp;
-  int highest_version;
+  struct dirent *dp;
+  int the_highest;
   int this_version;
-  int file_name_length;
+  size_t file_name_length;
 
   dirp = opendir (dirname);
   if (!dirp)
     return 0;
 
-  highest_version = 0;
+  the_highest = 0;
   file_name_length = strlen (filename);
 
   while ((dp = readdir (dirp)) != 0)
     {
-      if (!REAL_DIR_ENTRY (dp) || NLENGTH (dp) <= file_name_length + 2)
+      if (!REAL_DIR_ENTRY (dp) || NAMLEN (dp) <= file_name_length + 2)
 	continue;
 
       this_version = version_number (filename, dp->d_name, file_name_length);
-      if (this_version > highest_version)
-	highest_version = this_version;
+      if (this_version > the_highest)
+	the_highest = this_version;
     }
 
   closedir (dirp);
-  return highest_version;
+  return the_highest;
 }
 
 
@@ -209,12 +193,11 @@ highest_version (filename, dirname)
    are no backups, or only a simple backup, return 0. */
 
 static int
-max_version (pathname)
-     char *pathname;
+max_version (char *pathname)
 {
   char *p;
   char *filename;
-  int pathlen = strlen (pathname);
+  size_t pathlen = strlen (pathname);
   int version;
 
   p = pathname + pathlen - 1;
@@ -223,12 +206,12 @@ max_version (pathname)
 
   if (*p == '/')
     {
-      int dirlen = p - pathname;
+      size_t dirlen = (size_t) (p - pathname);
       char *dirname;
 
       filename = p + 1;
       dirname = xmalloc (dirlen + 1);
-      strncpy (dirname, pathname, (dirlen));
+      strncpy (dirname, pathname, dirlen);
       dirname[dirlen] = '\0';
       version = highest_version (filename, dirname);
       free (dirname);
@@ -245,21 +228,21 @@ max_version (pathname)
    value of VERSION_CONTROL. */
 
 static char *
-generate_backup_filename (version_control, pathname)
-     enum backup_mode version_control;
-     char *pathname;
+generate_backup_filename (
+			   enum backup_mode the_mode,
+			   char *pathname)
 {
   int last_numbered_version;
   char *backup_name;
 
-  if (version_control == none)
+  if (the_mode == none)
     return 0;
 
-  if (version_control == simple)
+  if (the_mode == simple)
     return simple_backup_name (pathname);
 
   last_numbered_version = max_version (pathname);
-  if (version_control == numbered_existing
+  if (the_mode == numbered_existing
       && last_numbered_version == 0)
     return simple_backup_name (pathname);
 
@@ -286,15 +269,12 @@ static struct version_control_values values[] =
   {unknown, 0}			/* Initial, undefined value. */
 };
 
-
-extern char *getenv ();
-
 /* Determine the value of `version_control' by looking in the
    environment variable "VERSION_CONTROL".  Defaults to
    numbered_existing. */
 
 enum backup_mode
-version_control_value ()
+version_control_value (void)
 {
   char *version;
   struct version_control_values *v;
@@ -318,7 +298,7 @@ version_control_value ()
 /* Initialize information used in determining backup filenames. */
 
 void
-initialize_backups ()
+initialize_backups (void)
 {
   char *v = getenv ("SIMPLE_BACKUP_SUFFIX");
 
@@ -330,8 +310,8 @@ initialize_backups ()
   version_control = version_control_value ();
   if (version_control == unknown)
     {
-      fprintf (stderr, "indent:  Strange version-control value\n");
-      fprintf (stderr, "indent:  Using numbered-existing\n");
+      fprintf (stderr, "%s:  Strange version-control value\n", progname);
+      fprintf (stderr, "%s:  Using numbered-existing\n", progname);
       version_control = numbered_existing;
     }
 #endif /* !NODIR */
@@ -342,8 +322,7 @@ initialize_backups ()
    See the description at the beginning of the file for details. */
 
 void
-make_backup (file)
-     struct file_buffer *file;
+make_backup (struct file_buffer *file)
 {
   int fd;
   char *backup_filename;
@@ -352,14 +331,14 @@ make_backup (file)
   backup_filename = generate_backup_filename (version_control, file->name);
   if (!backup_filename)
     {
-      fprintf (stderr, "indent: Can't make backup filename of %s", file->name);
+      fprintf (stderr, "%s: Can't make backup filename of %s", progname, file->name);
       exit (system_error);
     }
 
   fd = creat (backup_filename, 0666);
   if (fd < 0)
     fatal ("Can't open backup file %s", backup_filename);
-  size = write (fd, file->data, file->size);
+  size = (unsigned) write (fd, file->data, file->size);
   if (size != file->size)
     fatal ("Can't write to backup file %s", backup_filename);
 
